
# q-expansion of newform 315.8.a.c, downloaded from the LMFDB on 09 November 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 315
weight = 8
poly_data = [-11, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0], 1], [[0, 2], 1]]

hecke_ring_character_values = None
aps_data = [[-8, 1], [0, 0], [-125, 0], [-343, 0], [3953, -380], [-8909, 418], [1199, 2210], [-1806, -5542], [-6922, 10690], [63449, -23772], [126384, 22554], [-132930, 43638], [55960, 37354], [473786, 24578], [-135637, -56742], [633896, -65224], [680060, 170640], [-906840, -11334], [-1094656, -506344], [747464, -222048], [3584894, -212396], [-3971487, -187504], [152356, -939444], [8971764, 247538], [2129037, 680782], [4229086, -2258654], [-7521079, -462442], [379246, 1205004], [-19782299, 28272], [-19712690, 916854], [7809692, 2195992], [3371962, 175078], [28935764, 3290990], [9868378, 97976], [-33839738, -3091996], [-43259847, 3155932], [14940402, -8852248], [2740434, 5699264], [-49957425, -7143998], [-14642825, 9905174], [-122859244, -3460128], [-84189984, 3501208], [87343217, 9834508], [-144534980, 792666], [-4118628, -9215594], [-33240532, 5302916], [-93992943, 6363120], [-136396455, 1708346], [-170818925, 22855782], [102191592, -31333306], [25617928, 31059880], [-237831493, 19395844], [68600566, -22783442], [76987010, -7973216], [98564982, -29558644], [-306296090, 1154584], [342733582, 7234524], [63684368, 22374162], [191816018, -767756], [203230645, 80738764], [64267259, -56709786], [405262163, 15106350], [112923099, 53448846], [27787310, -5938186], [74056575, 112937206], [37997730, -64395548], [209180292, -9135600], [-252856998, 56719672], [-277432590, -156657338], [-33559218, 44937274], [-1030063821, -41102738], [-217669288, 188488104], [1388694965, -38812722], [-312870100, 128293086], [595438140, 28293200], [-766406564, -8451604], [437488551, 95364952], [308880153, -191977186], [2042287879, -14198848], [229115378, 95692156], [-1644059856, -85826392], [-1865782123, -155712780], [1206150661, 183079736], [-2686524066, 63266504], [353081022, 246640944], [1305336366, -64417674], [-1011072287, 129342236], [-313776964, -313942810], [-681652704, -141330064], [-1514943016, 403463014], [51349817, -435840734], [-3483520350, 165199430], [-1803192794, -60586480], [2184529699, -393863308], [-2481132757, -70952684], [1768808801, 520858826], [585278062, 334684424], [-1713257450, 237092634], [-256964724, -234319592], [-2939530721, -629675476], [-5148391464, -131987870], [-569028384, -450493576], [262997540, -418406256], [-2938646986, -12552032], [3005093964, 169814164], [-734330629, -441134130], [-2113704144, -1027830476], [-5836069635, -326554046], [-450733995, -892485640], [5360471186, 394422088], [1578657455, 579284238], [-854534846, 1367772024], [-5213885670, -443571988], [-2884985858, -539032626], [-9462881637, 95923292], [-4695964786, -579738048], [-5233990655, -1025174074], [-5397239344, 785145364], [-3436665322, -794951050], [2892059685, -323185760], [-2144735608, 1598099540], [7502342004, -409590206], [1277160997, 1762195466], [1296411960, 1185461994], [7486089172, 458850380], [3324660061, -693291004], [1047105135, -1239974000], [-3183861670, -1306494470], [-6952823664, 342830860], [3127087727, -277626510], [-3614017481, -249225892], [-6859794408, 408498988], [868742265, -1092744104], [-10538467008, -215173668], [1480027834, 2806454342], [3218349658, 3233704136], [2854117365, -1770227626], [4792845413, -1423571866], [-12310707473, -298233474], [10364800615, -820352720], [13227790742, 1560670340], [21432201455, 1090084404], [-27812529680, 18518726], [6433084138, 947427648], [-5544220224, -1437127978], [-3516022506, -162678962], [201699106, -202224432], [2601495882, -1567208824], [-3551823348, 2454273344], [-3737344372, 181705850], [-1180117862, 933176554], [-9606423856, 20717912], [-8419713004, 2944906238], [14048719456, -1020443216], [11104937914, -3777219354], [-9139600104, 1332899036], [8291480157, 2778241124], [30297061008, 955273236], [12235725525, 86387338], [-9123570096, 3194489542], [-18397083308, 1513112586], [-2020174316, -6472372498], [-20499081514, 3110591300], [741893268, 4416511470], [-40862746382, -666831362], [-4490372327, -6279572790], [3762884024, -2156236016], [-38157107059, -1796445990], [-10885469291, -8111576572], [-14938470142, 3179970900], [-21858377048, 4138754504], [-34867601434, 1024036268], [-23414369951, -82906328], [-3116940138, 2966523060], [-3783549150, -5339316650], [-4620448156, 2605386888], [-30250445629, 583988008], [-11548424753, -2812050192], [-1413993772, 5798585772], [-32851052704, -790575178], [4579362540, -3487943262], [20987399428, -554007110], [-36718344458, -4883584690], [-31363107674, -7062135296], [-61341230732, -268004870], [60919720086, -3993716598], [-1795394948, -5942747748], [-29191955351, -5982914386], [27517424433, 5904717696], [48663878040, 7895101134], [-2448764626, -3620917176], [51540166254, 2040953842], [54132834141, 154086448], [27980981684, 5967360802], [34296936972, -4260076444], [-47500271067, -6599528190], [-76925670921, 2574677004], [-31111953208, -8504145910], [42039225759, 10765531050], [81360748151, 1187061106], [-23911948654, -4374678848], [-16328714326, -6795031216], [-47575201802, 7168010884], [3244588478, 151255666], [41479112322, -13976031026], [17381374819, 16738547970], [66124589894, 8208710808], [39452078564, -8523126758], [-39950040246, -8993654592], [-34617172568, 1086264570], [95805667176, -5598277130], [35488607054, -4636688448], [-77244381496, -5506060006], [1935041539, 15108621310], [-12484046146, -5984994052], [52784113092, 14034472150], [21209698378, 1522843574], [-40276973936, 1467052102], [10268045010, -3038461118], [-9246816290, 9523010024], [72666028290, -14326812364], [76126831106, -3431786208], [12626819707, -4869453956], [-52155413786, -10247905476], [9268510587, -1409665874], [-115106668770, 6081829660], [-7430443405, -19904346], [100173719157, 6677350864], [21853469849, -3811903434], [166601819396, -3752586884], [110709103348, 9222729782], [24747883046, 10209557452], [57008160176, -15064817372], [-67792302353, -281023652], [164687802045, 12683015226], [153781818655, -8745909098], [7509147524, 9124350238], [66072890776, -8829396244], [61731837275, -2063261280], [47126830466, 24703229626], [101124038208, 14331417320], [188412752944, 11002079124], [-162453453123, 5790805450], [-74860941834, -5548034488], [-61112308163, -26321591822], [-11838984398, 19820639094], [87067150024, -3150357724], [65386344470, 4012564404], [19644332273, 19159138916], [-26728560354, 39025094946], [229748698058, -8675509226], [-94321904302, 2016337700], [-24558576552, -22998271892], [-114838299424, -8481315828], [54894722950, -29589976516], [-152742860227, 10690192716], [-58169771850, -21127795084], [-119105017140, -6279884112], [162795209589, -710638890], [-53393595349, 23513225810], [91756521026, -23245464902], [165467993830, -143703468], [171549092952, -18819893676], [-332146926977, -8541545738], [126759202274, -2794820500], [117491443998, -21952548054], [-9597747161, 14415336768], [41974991082, -32653798672], [-117088143040, -33535785824], [-202821733938, 241284758], [-103863582404, -43006197002], [151970746016, -12376654630], [-119568969355, -3360343414], [97374879252, 34940388384], [-137056821110, -30130280492], [123298003053, -19797945158], [-45498058780, 62142383352], [27934069978, 21035676376], [-94770854199, 31343416368], [-273912802650, 14434696244], [-198006551673, -10152501982], [202505487089, -23014533776], [-108441416656, -38470336800], [-139463745114, 46962850628], [186222909485, 13891253498], [123760069877, -52311635712], [45562726878, -25939706638], [15941559310, 37018726702], [278171968184, 7919510448], [200456064472, 41245876264], [-45003651641, 28555525012], [193931291553, 25984454178], [-59333357940, -25291640386], [-413781308760, -11462831006], [293984950490, 13207813166], [-176033522100, 32688057270], [-206226142826, -4528163756], [36912890845, 17091295214], [17885072940, -74215788460], [204313151393, 93832934], [7580915201, -52048619702], [-38296616386, 43287266520], [170528895135, -60422972760], [-7196050024, -37102693384], [-52470244555, -29597819764], [76534106670, 28626113118], [264440724406, 66834888514], [-167354189416, 5783883768], [-89728329936, 33541814188], [34396718572, 55109372980], [-241076113904, -31077382644], [90198219797, -47272515536], [-54797996062, 10086841480], [-253555475110, -22823898464], [-51348380058, 12293973806], [-340197379060, 37875801730], [-280209561802, 28848645256], [126393615765, -47755970516], [121376311673, -60808454898], [8548601507, -1511712656], [15512524950, 65320284996], [-436419860620, -9316688762], [-234041944994, -48985488684], [-71557809344, 46511169238], [584198677378, 33469386320], [-593221101184, 21554763188], [-139210201715, 10010230200], [544421656012, -9285562740], [-286222034820, -25164830778], [-479852907472, 40071798270], [6288860566, -121738080952], [148636520136, -122063704286], [225488466586, -46019239548], [588377229613, 23992109010], [-560720895564, 67386917508], [-548612484305, 61417530842], [81424579365, 32399649300], [163830830530, 33728116704], [-989315438262, -2534337018], [233661964101, 632931810], [-525524361090, -17970276984], [-55355404716, 39963279760], [-833884994074, 31960371882], [-229366015473, 34379027840], [-1131261795874, 5032034386], [547554530708, -1782856944], [257693427470, 54390205436], [-176120031708, 70492093022], [-799685978614, 58489718466], [-915124845961, 24181211850], [-328657159748, 82392021720], [191122823112, -19856219138], [405841429223, -89087358080], [737701225797, -29431997110], [419348955905, 115574209180], [669272114606, 64603396476], [-587161439326, -68341358892], [799733041110, -43689784146], [-746486900562, -41777932222], [149477230798, 83766627120], [532766834304, -9861516534], [531097393404, 8192343272], [-414685448996, 15372013190], [-221791070339, -18776083428], [-394037784484, -94639988000], [-870171027102, 9992204542], [-391315225017, -72000868062], [-167775340714, 43828544818], [-183366550907, 98681862632], [-204297877995, 142778508044], [-743957233832, 40042364666], [-144974745868, -774962912], [-719083958563, -13867993576], [138781575854, -129944752456], [-705267278030, -63428747552], [137986185104, 138392317388], [-63525983703, 158432814108], [-396690296485, 134879245946], [-151196088982, 45471312456], [728272660511, -128370062950], [186671686166, -218977924844], [-121952073698, -174291386252], [128756971896, 109769059840], [-883535421282, 49884315988], [215345363900, 127403183170], [375693111015, 36133675014], [595526947259, -55746845876], [-564025459365, 4649943478], [895402964996, -41657465532], [719144793197, -69180953594], [-11287090255, 4288091960], [-606910330989, 35109610066], [1681526549695, 6025209440], [-254640676348, 27929262712], [-162457997155, -157151084196], [-950301110744, -86183290412], [-413517655980, 203423282970], [-174482870932, 92391141866], [970438678735, 67980550036], [-133671709850, 90661624264], [-780701922428, 4549295194], [33584493760, 130418199624], [-1201969648566, -88334449754], [1086448470435, -12761963878], [-25272752484, 28212231592], [-783781699014, 110127340648], [-929345623346, 98956480256], [371612833193, -198920768960], [446188163092, 22285855920], [101834827932, -8535534696], [-811295242475, 91510109266], [-324721185470, 68482300772], [464482406586, -32892955914], [1059007292852, 177857239720], [1236180547090, 70236595818], [519905852472, -2360522752], [-816579018595, -42031657380], [-558080577940, -105035963852], [-549361039214, -21669827076], [-862811315217, 27561625370], [300225677859, -22954871452], [250474151746, -136905763640], [-895384596870, -221365500288], [1664758602084, 36738789748], [1321432506719, -125363515566], [-534120611289, -125542541146], [-365769818564, 241599059596], [307569445038, -147261990324], [1844204431468, -101159236518], [838689819896, -147149972878]]
